from django.test import TestCase


class TestSetUp(TestCase):
    """Class with setup and teardown for tests in XIS"""

    def setUp(self):
        """Function to set up necessary data for testing"""

        # globally accessible data sets

        self.supplemental_api_endpoint = 'http://openlxp-xis:8020' \
                                         '/api/supplemental-data/'

        self.receive_email_list = ['receiver1@openlxp.com',
                                   'receiver1@openlxp.com']
        self.sender_email = "sender@openlxp.com"

        self.Subject = 'Test Email'
        self.Email_Content = 'Here is the email content'
        self.Signature = 'Email Signature'
        self.Email_Us = 'sample_email@openlxp.com'
        self.FAQ_URL = 'https://sample.url.com'
        self.Unsubscribe_Email_ID = 'unsubscribe_email@openlxp.com'

        return super().setUp()

    def tearDown(self):
        return super().tearDown()



# Generated by Django 3.2.3 on 2022-01-24 18:19

from django.db import migrations, models
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='EmailConfiguration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('Subject', models.CharField(default='OpenLXP Conformance Alerts', max_length=200)),
                ('Email_Content', models.TextField(blank=True, max_length=200, null=True)),
                ('Signature', models.TextField(max_length=200)),
                ('Email_Us', models.EmailField(help_text='Enter email address', max_length=254)),
                ('FAQ_URL', models.CharField(max_length=200)),
                ('Unsubscribe_Email_ID', models.EmailField(help_text='Enter email address', max_length=254)),
                ('Logs_Type', models.CharField(choices=[('attachment', 'ATTACHMENT'), ('message', 'MESSAGE')], help_text='Check readme files of the components before choosing the  log type', max_length=200)),
                ('HTML_File', models.FileField(help_text='Check sample HTML files in the readme files of the components', upload_to='HTML_Files')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ReceiverEmailConfiguration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('email_address', models.EmailField(help_text='Enter email personas addresses to send log data', max_length=254, unique=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SenderEmailConfiguration',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('sender_email_address', models.EmailField(help_text='Enter sender email address to send log data from', max_length=254)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.contrib import admin

from .models import ReceiverEmailConfiguration, SenderEmailConfiguration, \
    EmailConfiguration


@admin.register(ReceiverEmailConfiguration)
class ReceiverEmailConfigurationAdmin(admin.ModelAdmin):
    list_display = ('email_address',)


@admin.register(SenderEmailConfiguration)
class SenderEmailConfigurationAdmin(admin.ModelAdmin):
    list_display = ('sender_email_address',)


@admin.register(EmailConfiguration)
class EmailConfigurationAdmin(admin.ModelAdmin):
    list_display = ('Subject',
                    'Email_Content',
                    'Signature',
                    'Email_Us',
                    'FAQ_URL',
                    'Unsubscribe_Email_ID',
                    'Logs_Type',
                    'HTML_File',
                    )



from django.db import models
from django.forms import ValidationError
from django.urls import reverse
from model_utils.models import TimeStampedModel

from openlxp_notifications.management.utils.notification import \
    email_verification


class ReceiverEmailConfiguration(TimeStampedModel):
    """Model for Receiver Email Configuration """

    email_address = models.EmailField(
        max_length=254,
        help_text='Enter email personas addresses to send log data',
        unique=True)

    def get_absolute_url(self):
        """ URL for displaying individual model records."""
        return reverse('Configuration-detail', args=[str(self.id)])

    def __str__(self):
        """String for representing the Model object."""
        return f'{self.id}'

    def save(self, *args, **kwargs):
        email_verification(self.email_address)
        return super(ReceiverEmailConfiguration, self).save(*args, **kwargs)


class SenderEmailConfiguration(TimeStampedModel):
    """Model for Sender Email Configuration """

    sender_email_address = models.EmailField(
        max_length=254,
        help_text='Enter sender email address to send log data from')

    def save(self, *args, **kwargs):
        if not self.pk and SenderEmailConfiguration.objects.exists():
            raise ValidationError('There is can be only one '
                                  'SenderEmailConfiguration instance')
        return super(SenderEmailConfiguration, self).save(*args, **kwargs)


class EmailConfiguration(TimeStampedModel):
    """Model for Email Configuration """

    LOG_TYPE_CHOICES = (
        ('attachment', 'ATTACHMENT'),
        ('message', 'MESSAGE'),
    )

    Subject = models.CharField(max_length=200,
                               default='OpenLXP Conformance Alerts')

    Email_Content = models.TextField(max_length=200, null=True, blank=True)

    Signature = models.TextField(max_length=200)

    Email_Us = models.EmailField(max_length=254,
                                 help_text='Enter email address')

    FAQ_URL = models.CharField(max_length=200)

    Unsubscribe_Email_ID = models.EmailField(max_length=254,
                                             help_text='Enter email address')
    Logs_Type = models.CharField(max_length=200, choices=LOG_TYPE_CHOICES,
                                 help_text='Check readme files of the '
                                           'components before choosing the '
                                           ' log type')

    HTML_File = models.FileField(upload_to='HTML_Files',
                                 help_text='Check sample HTML files in the '
                                           'readme files of the components')

    def get_absolute_url(self):
        """ URL for displaying individual model records."""
        return reverse('Configuration-detail', args=[str(self.id)])

    def __str__(self):
        """String for representing the Model object."""
        return f'{self.id}'

    def save(self, *args, **kwargs):
        if not self.pk and EmailConfiguration.objects.exists():
            raise ValidationError('There is can be only one '
                                  'EmailConfiguration instance')
        return super(EmailConfiguration, self).save(*args, **kwargs)



from django.test import SimpleTestCase, tag

from django.core.exceptions import ValidationError

from openlxp_notifications.models import (ReceiverEmailConfiguration,
                                          SenderEmailConfiguration,
                                          EmailConfiguration)
from unittest.mock import patch


@tag('unit')
class ModelTests(SimpleTestCase):

    def test_create_sender_email_config(self):
        """Test that creating a new Sender Email Configuration entry is
        successful with defaults """
        sender_email_address = 'example@test.com'

        sender_email_Config = SenderEmailConfiguration(
            sender_email_address=sender_email_address)

        self.assertEqual(sender_email_Config.sender_email_address,
                         sender_email_address)

    def test_create_two_sender_email_configuration(self):
        """Test that trying to create more than one EMAIL Configuration throws
        ValidationError """
        with patch('openlxp_notifications.models.SenderEmailConfiguration'):
            with self.assertRaises(ValidationError):
                sender_email_address = 'example@test.com'
                sender_email_address1 = 'example@test.com'

                sender_email_Config = SenderEmailConfiguration(
                    sender_email_address=sender_email_address)

                sender_email_Config1 = SenderEmailConfiguration(
                    sender_email_address=sender_email_address1)

                sender_email_Config.save()
                sender_email_Config1.save()

    def test_create_receiver_email_config(self):
        """Test that creating a new Receiver Email Configuration entry is
        successful with defaults """
        email_address = 'example@test.com'

        receiver_email_Config = ReceiverEmailConfiguration(
            email_address=email_address)

        self.assertEqual(receiver_email_Config.email_address,
                         email_address)

    def test_create_email_config(self):
        """Test that creating a Email Configuration entry is
        successful with defaults """
        Subject = 'Notifications'
        Email_Content = 'Please find the email'
        Signature = 'OpenLXP'
        Email_Us = 'example@test.com'
        FAQ_URL = 'https.abc.xyz'
        Unsubscribe_Email_ID = 'example@test.com'
        Logs_Type = 'Message'

        email_config = EmailConfiguration(
            Subject=Subject, Email_Content=Email_Content,
            Signature=Signature, Email_Us=Email_Us, FAQ_URL=FAQ_URL,
            Unsubscribe_Email_ID=Unsubscribe_Email_ID, Logs_Type=Logs_Type)

        self.assertEqual(email_config.Subject,
                         Subject)
        self.assertEqual(email_config.Email_Content,
                         Email_Content)
        self.assertEqual(email_config.Signature,
                         Signature)
        self.assertEqual(email_config.Email_Us,
                         Email_Us)
        self.assertEqual(email_config.FAQ_URL,
                         FAQ_URL)
        self.assertEqual(email_config.Unsubscribe_Email_ID,
                         Unsubscribe_Email_ID)
        self.assertEqual(email_config.Logs_Type,
                         Logs_Type)

    def test_create_two_email_configuration(self):
        """Test that trying to create more than one EMAIL Configuration throws
        ValidationError """
        with patch('openlxp_notifications.models.EmailConfiguration'):
            with self.assertRaises(ValidationError):
                Subject = 'Notifications'
                Email_Content = 'Please find the email'
                Signature = 'OpenLXP'
                Email_Us = 'example@test.com'
                FAQ_URL = 'https.abc.xyz'
                Unsubscribe_Email_ID = 'example@test.com'
                Logs_Type = 'Message'

                emailConfig = EmailConfiguration(
                    Subject=Subject, Email_Content=Email_Content,
                    Signature=Signature, Email_Us=Email_Us, FAQ_URL=FAQ_URL,
                    Unsubscribe_Email_ID=Unsubscribe_Email_ID,
                    Logs_Type=Logs_Type)

                Subject1 = 'Notifications'
                Email_Content1 = 'Please find the email'
                Signature1 = 'OpenLXP'
                Email_Us1 = 'example@test.com'
                FAQ_URL1 = 'https.abc.xyz'
                Unsubscribe_Email_ID1 = 'example@test.com'
                Logs_Type1 = 'Message'

                emailConfig2 = EmailConfiguration(
                    Subject=Subject1, Email_Content=Email_Content1,
                    Signature=Signature1, Email_Us=Email_Us1, FAQ_URL=FAQ_URL1,
                    Unsubscribe_Email_ID=Unsubscribe_Email_ID1,
                    Logs_Type=Logs_Type1)

                emailConfig.save()
                emailConfig2.save()



import logging
from unittest.mock import patch

from ddt import ddt
from django.test import tag

from openlxp_notifications.management.commands.conformance_alerts import \
    send_log_email, send_log_email_with_msg
from openlxp_notifications.models import (ReceiverEmailConfiguration,
                                          SenderEmailConfiguration,
                                          EmailConfiguration)

from .test_setup import TestSetUp

logger = logging.getLogger('dict_config_logger')


@tag('unit')
@ddt
class CommandTests(TestSetUp):

    # Test cases for conformance_alerts

    def test_send_log_email(self):
        """Test for function to send emails of log file to personas with
        attachment"""
        with patch('openlxp_notifications.management.commands.'
                   'conformance_alerts.ReceiverEmailConfiguration') \
                as receive_email_cfg, \
                patch(
                    'openlxp_notifications.management.commands.'
                    'conformance_alerts.SenderEmailConfiguration') \
                as sender_email_cfg, \
                patch(
                    'openlxp_notifications.management.commands.'
                    'conformance_alerts.EmailConfiguration') \
                as email_cfg, \
                patch(
                    'openlxp_notifications.management.commands.'
                    'conformance_alerts.send_notifications',
                    return_value=None
                ) as mock_send_notification:
            receive_email = ReceiverEmailConfiguration(
                email_address=self.receive_email_list)
            receive_email_cfg.first.return_value = receive_email

            send_email = SenderEmailConfiguration(
                sender_email_address=self.sender_email)
            sender_email_cfg.first.return_value = send_email

            email_config = EmailConfiguration(
                Subject=self.Subject, Email_Content=self.Email_Content,
                Signature=self.Signature, Email_Us=self.Email_Us,
                FAQ_URL=self.FAQ_URL,
                Unsubscribe_Email_ID=self.Unsubscribe_Email_ID,
                Logs_Type='ATTACHMENT', HTML_File='HTML_Files/My_Html.html')

            email_cfg.first.return_value = email_config

            send_log_email()
            self.assertEqual(mock_send_notification.call_count, 1)

    def test_send_log_email_with_msg(self):
        """Test for function to send emails of log file to personas with
        message"""
        with patch('openlxp_notifications.management.commands.'
                   'conformance_alerts.EmailConfiguration') \
                as email_cfg, \
                patch(
                    'openlxp_notifications.management.commands.'
                    'conformance_alerts.send_notifications_with_msg',
                    return_value=None
                ) as mock_send_notifications_with_msg:
            email_config = EmailConfiguration(
                Subject=self.Subject, Email_Content=self.Email_Content,
                Signature=self.Signature, Email_Us=self.Email_Us,
                FAQ_URL=self.FAQ_URL,
                Unsubscribe_Email_ID=self.Unsubscribe_Email_ID,
                Logs_Type='MESSAGE', HTML_File='HTML_Files/My_Html.html')

            email_cfg.first.return_value = email_config

            send_log_email_with_msg(self.receive_email_list, self.sender_email,
                                    'Message')
            self.assertEqual(mock_send_notifications_with_msg.call_count, 1)
